1 VERSIONINFO
 FILEVERSION 1,1,1,0
 PRODUCTVERSION 1,1,1,0
 FILEFLAGSMASK 0x0L
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x0L
 FILETYPE 0x2L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904b0"
        BEGIN
            VALUE "CompanyName", "Andi Stone Media"
            VALUE "FileDescription", "Scene As Transition"
            VALUE "FileVersion", "1.1.1"
            VALUE "InternalName", "scene-as-transition"
            VALUE "LegalCopyright", "(C) Andi Stone Media"
            VALUE "OriginalFilename", "scene-as-transition"
            VALUE "ProductName", "Scene As Transition"
            VALUE "ProductVersion", "1.1.1"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1200
    END
END
